<?php
/*
Plugin Name: Ultimate tag cloud widget
Plugin URI: http://www.0x539.se/wordpress
Description: This plugin aims to be the most configurable tag cloud widget out there, able to suit all your wierd tag cloud needs.
Version: 1.3.2
Author: Rickard Andersson
Author URI: http://www.0x539.se
License: GPLv2
*/

/*
This program is free software; you can redistribute it and/or modify 
it under the terms of the GNU General Public License as published by 
the Free Software Foundation; version 2 of the License.

This program is distributed in the hope that it will be useful, 
but WITHOUT ANY WARRANTY; without even the implied warranty of 
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the 
GNU General Public License for more details. 

You should have received a copy of the GNU General Public License 
along with this program; if not, write to the Free Software 
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA 
*/

// Default values if the widget isn't configured properly
DEFINE("UTCW_DEFAULT_TITLE",          __("Tag Cloud", 'utcw'));
DEFINE("UTCW_DEFAULT_ORDER",          "name");
DEFINE("UTCW_DEFAULT_SIZE_FROM",      10);
DEFINE("UTCW_DEFAULT_SIZE_TO",        30);
DEFINE("UTCW_DEFAULT_MAX",            45);
DEFINE("UTCW_DEFAULT_TAXONOMY",       "post_tag");
DEFINE("UTCW_DEFAULT_REVERSE",        false);
DEFINE("UTCW_DEFAULT_COLOR",          "none");
DEFINE("UTCW_DEFAULT_LETTER_SPACING", "normal");
DEFINE("UTCW_DEFAULT_WORD_SPACING",   "normal");
DEFINE("UTCW_DEFAULT_CASE",           "off");
DEFINE("UTCW_DEFAULT_CASE_SENSITIVE", false);
DEFINE("UTCW_DEFAULT_MINIMUM",		  1);

// Allowed values for miscellanious options in the widget  
$utcw_allowed_orders     = array('random', 'name', 'slug', 'id', 'color', 'count');
$utcw_allowed_taxonomys  = array('post_tag', 'category');
$utcw_allowed_colors     = array('none', 'random', 'set', 'span');
$utcw_allowed_cases      = array('lowercase', 'uppercase', 'capitalize', 'off');


/**
 * Ultimate tag cloud widget class
 * @package UTCW
 * @author Rickard Andersson <rickard@0x539.se>
 * @todo Also update the wordpress page 
 */
class UTCW extends WP_Widget {
  
  /**
   * Constructor
   * @return void
   */
  function UTCW() {
    $options = array('description' => __("Highly configurable tag cloud", 'utcw'));
    parent::WP_Widget(false, __('Ultimate Tag Cloud', 'utcw'), $options);    
  }
  
  /**
   * Action handler for the form in the admin panel
   * @param array $new_instance
   * @param array $old_instance
   * @retrn array
   */
  function update($new_instance, $old_instance) {

    global $utcw_allowed_orders, 
           $utcw_allowed_taxonomys,
           $utcw_allowed_colors,
           $utcw_allowed_cases;

      
    extract($new_instance);
      
    // Check all input values and set the default value if any value is invalid or empty
    $instance = $old_instance;
    $instance['title']          = strlen($title) > 0          ? apply_filters('widget_title', $title) : apply_filters('widget_title', UTCW_DEFAULT_TITLE);
    $instance['size_from']      = is_numeric($size_from)      ? $size_from : UTCW_DEFAULT_SIZE_FROM;
    $instance['size_to']        = is_numeric($size_to)        ? $size_to : UTCW_DEFAULT_SIZE_TO;
    $instance['max']            = is_numeric($max)            ? $max : UTCW_DEFALUT_MAX;
    $instance['letter_spacing'] = is_numeric($letter_spacing) ? $letter_spacing : UTCW_DEFAULT_LETTER_SPACING;
    $instance['word_spacing']   = is_numeric($word_spacing)   ? $word_spacing : UTCW_DEFAULT_WORD_SPACING;
    $instance['minimum'] 		= is_numeric($minimum) 		  ? $minimum : UTCW_DEFAULT_MINIMUM;
    $instance['reverse']        = ($reverse == "on");
    $instance['exclude']        = strlen($exclude) > 0        ? @explode(",", $exclude) : array();
    $instance['color_set']      = strlen($color_set) > 0      ? @explode(",", $color_set) : array();
    $instance['authors']        = array();
    $instance['case_sensitive'] = ($case_sensitive == "on");
	

    $instance['color_span_from']  = preg_match('/#([a-f0-9]{6}|[a-f0-9]{3})/i', $color_span_from) > 0 ? $color_span_from : ""; 
    $instance['color_span_to']    = preg_match('/#([a-f0-9]{6}|[a-f0-9]{3})/i', $color_span_to) > 0   ? $color_span_to : "";

    $instance['taxonomy']   = in_array($taxonomy, $utcw_allowed_taxonomys) ? $taxonomy : UTCW_DEFAULT_TAXONOMY;
    $instance['order']      = in_array($order, $utcw_allowed_orders)       ? $order : UTCW_DEFAULT_ORDER;
    $instance['color']      = in_array($color, $utcw_allowed_colors)       ? $color : UTCW_DEFAULT_COLOR;    
    $instance['case']       = in_array($case, $utcw_allowed_cases)         ? $case : UTCW_DEFAULT_CASE;
    
    // Only accept numeric authors (user ID)    
    if (is_array($authors)) {
      foreach ($authors as $author) {
        if (is_numeric($author)) {
          $instance['authors'][] = $author;
        }
      }      
    }
        
    // Remove spaces in the comma separated list 
    foreach ($instance['exclude'] as $key => $value) {
      $instance['exclude'][$key] = trim($value);
    }
    
    // Only allow hexadecimal color values in the format #ffffff and #fff
    foreach ($instance['color_set'] as $key => $color) {
      if (preg_match('/#([a-f0-9]{6}|[a-f0-9]{3})/i', $color) == 0) {
        unset($instance['color_set'][$key]);
      }
    }

    return $instance;
  }

  /**
   * Function for displaying the widget on the page
   * @param array $args
   * @param array $instance
   * @return void
   */
  function widget($args, $instance) {    
    do_utcw(array_merge($instance, $args));
  }  

  /**
   * Function for handling the widget control in admin panel
   * @param array $instance
   * @return void 
   */
  function form($instance) {
    
    // Get stored preferences
    $title            = esc_attr($instance['title']);
    $order            = esc_attr($instance['order']);
    $size_from        = esc_attr($instance['size_from']);
    $size_to          = esc_attr($instance['size_to']);
    $max              = esc_attr($instance['max']);
    $taxonomy         = esc_attr($instance['taxonomy']);
    $color            = esc_attr($instance['color']);
    $color_span_from  = esc_attr($instance['color_span_from']);
    $color_span_to    = esc_attr($instance['color_span_to']);
    $letter_spacing   = esc_attr($instance['letter_spacing']);
    $word_spacing     = esc_attr($instance['word_spacing']);
    $case             = esc_attr($instance['case']);
	$minimum		  = esc_attr($instance['minimum']);
    $exclude          = is_array($instance['exclude'])   ? $instance['exclude'] : array();
    $reverse          = $instance['reverse'];
    $authors          = is_array($instance['authors']) 	 ? $instance['authors'] : array();
    $color_set        = is_array($instance['color_set']) ? $instance['color_set'] : array();;
    $case_sensitive   = $instance['case_sensitive'];      

// Content of the widget settings form
?> 
<fieldset class="utcw">
<legend><?php _e("Data", 'utcw'); ?></legend>
  <?php _e("Authors:", 'utcw'); ?><br/>
<?php foreach (get_users_of_blog() as $user) : ?>
  <input type="checkbox" name="<?php echo $this->get_field_name('authors'); ?>[]" id="<?php echo $this->get_field_id('author_' . $user->ID); ?>" value="<?php echo $user->ID?>" <?php echo  in_array($user->ID, $authors, true) ? 'checked="checked"' : ""; ?> /> 
  <label for="<?php echo $this->get_field_id('author_' . $user->ID); ?>"><?php echo $user->user_login?></label><br/>
<?php endforeach; ?>
  <br/>
  <?php _e("Order:", 'utcw'); ?><br/>
  <input type="radio" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order_random'); ?>" value="random" <?php echo  $order == "random" ? ' checked="checked" ' : ""; ?>/>
  <label for="<?php echo $this->get_field_id('order_random'); ?>"><?php _e("Random", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order_name'); ?>" value="name" <?php echo  $order == "name" ? ' checked="checked"' : ""; ?>/>
  <label for="<?php echo $this->get_field_id('order_name'); ?>"><?php _e("By name", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order_slug'); ?>" value="slug" <?php echo  $order == "slug" ? ' checked="checked"' : ""; ?>/>
  <label for="<?php echo $this->get_field_id('order_slug'); ?>"><?php _e("By slug", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order_id'); ?>" value="id" <?php echo  $order == "id" ? ' checked="checked"' : ""; ?>/>
  <label for="<?php echo $this->get_field_id('order_id'); ?>"><?php _e("By id", 'utcw'); ?></label><br/>  
  <input type="radio" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order_color'); ?>" value="name" <?php echo  $order == "color" ? ' checked="checked"' : ""; ?>/>
  <label for="<?php echo $this->get_field_id('order_color'); ?>"><?php _e("By color", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('order'); ?>" id="<?php echo $this->get_field_id('order_count'); ?>" value="count" <?php echo  $order == "count" || strlen($order) == 0 ? ' checked="checked" ' : ""; ?>/>  
  <label for="<?php echo $this->get_field_id('order_count'); ?>"><?php _e("Count", 'utcw'); ?></label><br/>
  <input type="checkbox" name="<?php echo $this->get_field_name('reverse'); ?>" id="<?php echo $this->get_field_id('reverse'); ?>" <?php echo  $reverse === true ? ' checked="checked"' : ""; ?> />
  <label for="<?php echo $this->get_field_id('reverse'); ?>"><?php _e("Reverse order", 'utcw'); ?></label><br/>
  <input type="checkbox" name="<?php echo $this->get_field_name('case_sensitive'); ?>" id="<?php echo $this->get_field_id('case_sensitive'); ?>" <?php echo $case_sensitive === true ? ' checked="checked"' : ""; ?> />
  <label for="<?php echo $this->get_field_id('case_sensitive'); ?>"><?php _e("Case sensitive", 'utcw'); ?></label><br/>
  <br/>
  <label for="<?php echo $this->get_field_id('taxonomy'); ?>"><?php _e("Taxonomy:", 'utcw'); ?></label><br/>
  <select id="<?php echo $this->get_field_id('taxonomy'); ?>" name="<?php echo $this->get_field_name('taxonomy'); ?>">
    <option value="post_tag" <?php echo  $taxonomy == "post_tag" || strlen($taxonomy) == 0 ? 'selected="selected"' : ""?>><?php _e("Post Tags", 'utcw'); ?></option>
    <option value="category" <?php echo  $taxonomy == "category" ? 'selected="selected"' : ""?>><?php _e("Categories", 'utcw'); ?></option>
  </select><br/>
  <br/>
  <label for="<?php echo $this->get_field_id('minimum'); ?>"><?php _e("Minimum number of posts: ", 'utcw'); ?></label>
  <input type="text" name="<?php echo $this->get_field_name('minimum'); ?>" id="<?php echo $this->get_field_id('minimum'); ?>" value="<?php echo $minimum; ?>" /><br/>
  <small><?php _e("Tags with fewer posts than this will be automatically excluded.", 'utcw'); ?></small><br/>
  <br/>
  <label for="<?php echo $this->get_field_id('exclude'); ?>"><?php _e("Exclude these tags:", 'utcw'); ?></label><br/>
  <input type="text" name="<?php echo $this->get_field_name('exclude'); ?>" id="<?php echo $this->get_field_id('exclude'); ?>" value="<?php echo implode(",", $exclude); ?>" /><br/>
  <small><?php _e("Enter either the names or IDs of the tags you'd like to exclude.", 'utcw'); ?></small>
</fieldset>

<fieldset class="utcw">
<legend><?php _e("Appearance", 'utcw'); ?></legend>
  <label for="<?php echo $this->get_field_id('title');?>"><?php _e("Title:", 'utcw') ?></label><br/>
  <input type="text" id="<?php echo $this->get_field_id('title');?>" name="<?php echo $this->get_field_name('title');?>" value="<?php echo  strlen($title) > 0 ? $title : UTCW_DEFAULT_TITLE?>" /><br/>
  <br/>
  <?php _e("Tag size (in px):", 'utcw'); ?><br/>
  <label for="<?php echo $this->get_field_id('size_from'); ?>"><?php _e("From", 'utcw'); ?></label>
  <input type="text" name="<?php echo $this->get_field_name('size_from'); ?>" id="<?php echo $this->get_field_id('size_from'); ?>" size="3" value="<?php echo is_numeric($size_from) ? $size_from : UTCW_DEFAULT_SIZE_FROM?>" />
  <label for="<?php echo $this->get_field_id('size_to'); ?>"><?php _e("to", 'utcw'); ?></label>
  <input type="text" name="<?php echo $this->get_field_name('size_to'); ?>" id="<?php echo $this->get_field_id('size_to'); ?>" size="3" value="<?php echo is_numeric($size_to) ? $size_to : UTCW_DEFAULT_SIZE_TO?>" /><br/>
  <br/>
  <label for="<?php echo $this->get_field_id('max'); ?>"><?php _e("Max tags:", 'utcw'); ?></label><br/>
  <input type="text" name="<?php echo $this->get_field_name('max'); ?>" id="<?php echo $this->get_field_id('max'); ?>" value="<?php echo is_numeric($max) ? $max : UTCW_DEFAULT_MAX?>" /><br/>
  <br/>
  <?php _e("Coloring:", 'utcw') ?><br/>
  <input type="radio" name="<?php echo $this->get_field_name('color'); ?>" id="<?php echo $this->get_field_id('color_none'); ?>" value="none" <?php echo  $color == "none" || empty($color) ? 'checked="checked"' : ""; ?> onclick="javascript:utcw_change()" />  
  <label for="<?php echo $this->get_field_id('color_none'); ?>"><?php _e("None", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('color'); ?>" id="<?php echo $this->get_field_id('color_random'); ?>" value="random" <?php echo  $color == "random" ? 'checked="checked"' : ""; ?> onclick="javascript:utcw_change()" />
  <label for="<?php echo $this->get_field_id('color_random'); ?>"><?php _e("Totally random", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('color'); ?>" id="<?php echo $this->get_field_id('color_set'); ?>" value="set" <?php echo  $color == "set" ? 'checked="checked"' : ""; ?> onclick="javascript:utcw_change()" />
  <label for="<?php echo $this->get_field_id('color_set'); ?>"><?php _e("Random from preset values", 'utcw'); ?></label><br/>
  <div id="<?php echo $this->get_field_id('set_chooser'); ?>" <?php echo  $color != "set" ? 'class="utcw-hidden"' : ""; ?>>
    <input type="text" name="<?php echo $this->get_field_name('color_set'); ?>" id="<?php echo $this->get_field_id('color_set_chooser'); ?>" value="<?php echo  implode(",", $color_set) ?>" />
  </div>
  <input type="radio" name="<?php echo $this->get_field_name('color'); ?>" id="<?php echo $this->get_field_id('color_span'); ?>" value="span" <?php echo  $color == "span" ? 'checked="checked"' : ""; ?> onclick="javascript:utcw_change()" />
  <label for="<?php echo $this->get_field_id('color_span'); ?>"><?php _e("Spanning between values", 'utcw'); ?></label><br/>
  <div id="<?php echo $this->get_field_id('span_chooser'); ?>" <?php echo  $color != "span" ? 'class="utcw-hidden"' : ""; ?>>
    <?php _e("From", 'utcw'); ?> <input type="text" size="7" name="<?php echo $this->get_field_name('color_span_from'); ?>" id="<?php echo $this->get_field_id('color_span_from'); ?>" value="<?php echo $color_span_from?>" />
    <?php _e("to", 'utcw'); ?> <input type="text" size="7" name="<?php echo $this->get_field_name('color_span_to'); ?>"" id="<?php echo $this->get_field_id('color_span_to'); ?>" value="<?php echo $color_span_to?>" /> 
  </div>
  <br/>
  <?php _e("Spacing (in px):", 'utcw'); ?><br/>
  <label for="<?php echo $this->get_field_id('letter_spacing'); ?>"><?php _e("Between letters:", 'utcw'); ?></label>
  <input type="text" size="5" name="<?php echo $this->get_field_name('letter_spacing'); ?>" id="<?php echo $this->get_field_id?>" value="<?php echo strlen($letter_spacing) > 0 ? $letter_spacing : UTCW_DEFAULT_LETTER_SPACING?>" /><br/>
  <label for="<?php echo $this->get_field_id('word_spacing'); ?>"><?php _e("Between words:", 'utcw'); ?></label>
  <input type="text" size="5" name="<?php echo $this->get_field_name('word_spacing'); ?>" id="<?php echo $this->get_field_id('word_spacing'); ?>" value="<?php echo strlen($word_spacing) > 0 ? $word_spacing : UTCW_DEFAULT_WORD_SPACING?>" /><br/>
  <br/>
  <?php _e("Transform tags:", 'utcw'); ?><br/>
  <input type="radio" name="<?php echo $this->get_field_name('case'); ?>" id="<?php echo $this->get_field_id('text_transform_off'); ?>" value="off" <?php echo  ($case == "off" || empty($case)) ? ' checked="checked"' : ""; ?> /> 
  <label for="<?php echo $this->get_field_id('text_transform_off'); ?>"><?php _e("Off", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('case'); ?>" id="<?php echo $this->get_field_id('text_transform_lowercase'); ?>" value="lowercase" <?php echo  $case == "lowercase" ? ' checked="checked"' : ""; ?> />
  <label for="<?php echo $this->get_field_id('text_transform_lowercase'); ?>"><?php _e("To lowercase", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('case'); ?>" id="<?php echo $this->get_field_id('text_transform_uppercase'); ?>" value="uppercase" <?php echo  $case == "uppercase" ? ' checked="checked"' : ""; ?> />
  <label for="<?php echo $this->get_field_id('text_transform_uppercase'); ?>"><?php _e("To uppercase", 'utcw'); ?></label><br/>
  <input type="radio" name="<?php echo $this->get_field_name('case'); ?>" id="<?php echo $this->get_field_id('text_transform_capitalize'); ?>" value="capitalize" <?php echo  $case == "capitalize" ? ' checked="checked"' : ""; ?> />
  <label for="<?php echo $this->get_field_id('text_transform_capitalize'); ?>"><?php _e("Capitalize", 'utcw'); ?></label><br/>
</fieldset>
<?php
  }
}

/**
 * do_utcw - Prints markup for the widget
 * @param array $args   An array with widget settings. See {@link http://wordpress.org/extend/plugins/ultimate-tag-cloud-widget/other_notes/} for details on which options you can set.  
 * @return void 
 */

function do_utcw($args) {

  global $wpdb;
  
  extract($args);
  
  global $utcw_allowed_orders, 
         $utcw_allowed_taxonomys,
         $utcw_allowed_colors,
         $utcw_allowed_cases;
  
  // Parse settings from $instance and set default values where empty or invalid
  $title           = strlen($title) > 0           ? $title : UTCW_DEFAULT_TITLE;
  $size_from       = is_numeric($size_from)       ? $size_from : UTCW_DEFAULT_SIZE_FROM;
  $size_to         = is_numeric($size_to)         ? $size_to : UTCW_DEFAULT_SIZE_TO;
  $max             = is_numeric($max)             ? $max : UTCW_DEFAULT_MAX;
  $reverse         = is_bool($reverse)            ? $reverse : UTCW_DEFAULT_REVERSE;
  $minimum		   = is_numeric($minimum) 		  ? $minimum : UTCW_DEFAULT_MINIMUM;
  $authors         = is_array($authors)           ? $authors : array();
  $exclude         = is_array($exclude)           ? $exclude : array();
  $color_set       = is_array($color_set)         ? $color_set : array();
  $letter_spacing  = is_numeric($letter_spacing)  ? $letter_spacing . "px" : UTCW_DEFAULT_LETTER_SPACING;
  $word_spacing    = is_numeric($word_spacing)    ? $word_spacing . "px" : UTCW_DEFAULT_WORD_SPACING;
  $color_span_from = is_string($color_span_from)  ? $color_span_from : "";
  $color_span_to   = is_string($color_span_to)    ? $color_span_to : "";
  $case_sensitive  = is_bool($case_sensitive)     ? $case_sensitive : UTCW_DEFAULT_CASE_SENSITIVE;

  $order      = in_array($order, $utcw_allowed_orders)       ? $order : UTCW_DEFAULT_ORDER;
  $taxonomy   = in_array($taxonomy, $utcw_allowed_taxonomys) ? $taxonomy : UTCW_DEFAULT_TAXONOMY;
  $color      = in_array($color, $utcw_allowed_colors)       ? $color : UTCW_DEFAULT_COLOR;
  $case       = in_array($case, $utcw_allowed_cases)         ? $case : UTCW_DEFAULT_CASE;    
             
  // Fallback values
  $counts = array();
  $tag_array = array();

  // Build SQL query
  $q = "SELECT t.term_id, t.name, t.slug, COUNT(tr.term_taxonomy_id) AS `count` ";
  $q .= "FROM `$wpdb->posts` AS p ";
  $q .= "LEFT JOIN `$wpdb->term_relationships` AS tr ON tr.object_id = p.ID ";
  $q .= "LEFT JOIN `$wpdb->term_taxonomy` AS tt ON tt.term_taxonomy_id = tr.term_taxonomy_id ";
  $q .= "LEFT JOIN `$wpdb->terms` AS t ON t.term_id = tt.term_id ";
  $q .= "WHERE tt.taxonomy = '$taxonomy' ";
  $q .= "AND post_type = 'post' ";
  $q .= "AND post_status = 'publish' ";

  if (count($authors) > 0) {
    $q .= "AND post_author IN (" . implode(",", $authors) . ") ";
  }
  if (count($exclude) > 0) {
    if (is_array_numeric($exclude)) {
      $q .= "AND t.term_id NOT IN ('" . implode("', '", $exclude) . "') ";
    } else {
      $q .= "AND t.name NOT IN ('" . implode("', '", $exclude) . "') ";
    }
  }
  if (is_numeric($minimum)) {
  	$q .= "AND count >= $minimum ";
  }

  $q .= "GROUP BY tr.term_taxonomy_id ";
  $q .= "ORDER BY count DESC ";
  $q .= "LIMIT $max;";
  
  $tag_data = $wpdb->get_results($q);

  if (count($tag_data) > 0) {
  
    // Extract counts and create an array to work with
    foreach ($tag_data as $tag) {
      $counts[] = $tag->count;
      $tag_array[] = array(
        'term_id' => $tag->term_id, 
        'count' => $tag->count, 
        'slug' => $tag->slug,
        'name' => $tag->name,
        'link' => get_term_link(intval($tag->term_id), $taxonomy),
        'color' => ""
      );
    }
    
    // Highest and lowest values
    $min_count = min($counts);
    $max_count = max($counts);
    
    // Get the step size 
    $font_step = calc_step($min_count, $max_count, $size_from, $size_to);
    
    // Calculate sizes for all tags
    foreach ($tag_array as $key => $tag) {
      $tag_array[$key]['size'] = $size_from + ( ( $tag['count'] - $min_count ) * $font_step );
    }
    
    // Check the coloring preference, default is none 
    switch ($color) {
        
      // Just get an randomized value, who would ever use this?!
      case "random":
        foreach ($tag_array as $key => $tag) {
          $tag_array[$key]['color'] = sprintf("#%s%s%s", dechex(rand() % 255), dechex(rand() % 255), dechex(rand() % 255));
        }
        break;
        
      // Select a random value from the preset colors
      case "set": 
        if (is_array($color_set) && count($color_set) > 0) {
          foreach ($tag_array as $key => $tag) {
            $tag_array[$key]['color'] = $color_set[ array_rand($color_set) ];
          }
        }
        break;
      
      // Calculate colors in a span between two values 
      case "span":
        
        // Check the color format, #fff or #fffff
        if (strlen($color_span_from) == 4) {
          $red_from    = hexdec(sprintf("%s%s", $color_span_from[1], $color_span_from[1]));
          $green_from  = hexdec(sprintf("%s%s", $color_span_from[2], $color_span_from[2]));
          $blue_from   = hexdec(sprintf("%s%s", $color_span_from[3], $color_span_from[3]));
        } else {
          $red_from    = hexdec(substr($color_span_from, 1, 2));
          $green_from  = hexdec(substr($color_span_from, 3, 2));
          $blue_from   = hexdec(substr($color_span_from, 5, 2));
        }
        if (strlen($color_span_to) == 4) {
          $red_to    = hexdec(sprintf("%s%s", $color_span_to[1], $color_span_to[1]));
          $green_to  = hexdec(sprintf("%s%s", $color_span_to[2], $color_span_to[2]));
          $blue_to   = hexdec(sprintf("%s%s", $color_span_to[3], $color_span_to[3]));
        } else {
          $red_to    = hexdec(substr($color_span_to, 1, 2));
          $green_to  = hexdec(substr($color_span_to, 3, 2));
          $blue_to   = hexdec(substr($color_span_to, 5, 2));
        }
        
        // Calculate steps for all the colors. 
        $red_step   = calc_step($min_count, $max_count, $red_from, $red_to);
        $green_step = calc_step($min_count, $max_count, $green_from, $green_to);
        $blue_step  = calc_step($min_count, $max_count, $blue_from, $blue_to);
        
        // Iterate all tags and calculate their color
        foreach ($tag_array as $key => $tag) {
          $red    = round($red_from + ( ( $tag['count'] - $min_count ) * $red_step ));
          $green  = round($green_from + ( ( $tag['count'] - $min_count ) * $green_step ));
          $blue   = round($blue_from + ( ( $tag['count'] - $min_count ) * $blue_step ));
          
          $tag_array[$key]['color'] = sprintf("rgb(%s,%s,%s)", $red, $green, $blue); 
        }
        break;
    }
    
    // Check the ordering preference, default is name
    switch ($order) {
      case "random":
        shuffle($tag_array);
        break;
        
      case "count":
        usort($tag_array, 'utcw_cmp_count');
        break;
        
      case "slug";
        usort($tag_array, $case_sensitive === true ? 'utcw_cmp_slug' : 'utcw_icmp_slug');
        break;
        
      case "id":
        usort($tag_array, 'utcw_cmp_id');
        break;
        
      case "color":
        usort($tag_array, 'utcw_cmp_color');
        break;
        
      case "name":  
      default:
        usort($tag_array, $case_sensitive === true ? 'utcw_cmp_name' : 'utcw_icmp_name');
        break;
    }
    
    // Reverse the list if the user prefers it that way. Reversing an random sorted result seems correct. 
    if ($reverse === true) {
      $tag_array = array_reverse($tag_array);
    }      
  }

  switch ($case) {
    case 'uppercase':
      $text_transform = 'text-transform: uppercase;';
      break;
      
    case 'lowercase':
      $text_transform = 'text-transform: lowercase;';
      break;
      
    case 'capitalize':
      $text_transform = 'text-transform: capitalize;';
      break;
      
    default:
      $text_transform = '';
  }
      
  // Print the tag cloud content
  echo $before_widget;
  echo $before_title . $title . $after_title;
  
  printf('<div class="widget_tag_cloud" style="letter-spacing:%s;word-spacing:%s;%s">', 
      $letter_spacing, $word_spacing, $text_transform);
       
  foreach ($tag_array as $tag) {
    extract($tag);
    
    printf('<a class="tag-link-%s" href="%s" style="font-size:%spx;%s;" title="' . _n("%s topic", "%s topics", $count) . '">%s</a> ',
        $term_id, $link, $size, strlen($color) > 0 ? "color:$color" : "", $count, $name);
  }
  
  echo "</div>";
  
  echo $after_widget;
}

/**
 * Used to calculate how step size in spanning values
 * @param integer $min
 * @param integer $max
 * @param integer $from
 * @param integer $to
 * @return integer 
 */
function calc_step($min, $max, $from, $to) {
  
  // Thank you wordpress for this 
  $spread = $max - $min;
  if ( $spread <= 0 )
    $spread = 1;
  $font_spread = $to - $from;
  if ( $font_spread < 0 && $negative === false)
    $font_spread = 1; 
  $step = $font_spread / $spread;
  
  return $step;
}

load_plugin_textdomain('utcw', false, '/ultimate-tag-cloud-widget/language/');

// Register widget with wordpress
add_action('widgets_init', create_function('', 'return register_widget("UTCW");'));

//Register scripts and css with wordpress
wp_register_script('utcw-js', '/wp-content/plugins/ultimate-tag-cloud-widget/utcw.js', array('jquery'), "1.2", false);
wp_register_style('utcw-css', '/wp-content/plugins/ultimate-tag-cloud-widget/utcw.css', array(), "1.2", 'all');

/**
 * Action handler for wordpress init used to attach scripts and styles
 * @return void 
 */
function utcw_init() {
  wp_enqueue_script('utcw-js');
  wp_enqueue_style('utcw-css');   
}
add_action('init', 'utcw_init');


/*
 * Compare functions
 */
function utcw_cmp_count($a, $b) {
  if ($a['count'] == $b['count']) {
    return 0;
  } else {
    return ($a['count'] < $b['count']) ? -1 : 1;    
  }
}

function utcw_icmp_name($a, $b) {
  return strcasecmp($a['name'], $b['name']);
}

function utcw_cmp_name($a, $b) {
  return strcmp($a['name'], $b['name']);
}

function utcw_icmp_slug($a, $b) {
  return strcasecmp($a['slug'], $b['slug']);
}

function utcw_cmp_slug($a, $b) {
  return strcmp($a['slug'], $b['slug']);
}

function utcw_cmp_id($a, $b) {
  if ($a['term_id'] == $b['term_id']) {
    return 0;
  } else {
    return ($a['term_id'] < $b['term_id']) ? -1 : 1;    
  }  
}

function utcw_cmp_color($a, $b) {
  return strcasecmp($a['color'], $b['color']);
}

/**
 * Function for checking if every item within an array is a numeric value
 * @param arrau $array
 * @return boolean
 */
function is_array_numeric($array) {
  
  foreach ($array as $item) {
    if (!is_numeric($item)) {
      return false;
    }
  }
  
  return true;
  
}
